import rpy2.robjects.packages as rpackages
from .base import BaseManager

class RManager(BaseManager):

    def __init__(self, log_file=''):
        super(RManager, self).__init__(log_file)
        self.devtools = rpackages.importr('devtools')
        self.utils = rpackages.importr("utils")
    
    def install_version(self, package, version):
        self.devtools.install_version(package, version=version)
    
    def install(self, package, version, git):
        self.log_install(package, version, git)
        if package:
            if git:
                self.install_git(package)
            elif version:
                self.install_version(package, version)                    
            else:
                self.utils.install_packages(package)        
    
    def install_git(self, repo):
        self.devtools.install_git(repo)
    
    def install_list(self, package_list, extra_config={}):
        for package_item in package_list:
            package = package_item.get('name')
            version = package_item.get('version')
            git = package_item.get('git')
            self.install(package, version, git)



import os
import re
import click
from .managers.python import PyManager
from .managers.r import RManager
from .broker import BrokerConnector


@click.group()
def bus():
    pass


@bus.command()
@click.option('--language', '-l', help='Language name', type=click.Choice(['python', 'r']), required=True)
@click.option('--name', '-n', help='Package Name', required=True)
@click.option('--version', '-v', help='Package Version', default="")
@click.option('--git', '-g', help='Install from git', default=False, is_flag=True)
@click.option('--pip_paths', '-i', help='Pip paths', multiple=True, default=["pip"])
@click.option('--log_file', '-f', help='LogFile')
def install(language, name, version, git, pip_paths, log_file):
    if language == 'python':
        installer = PyManager(log_file)
        arg_list = [name, version, git, pip_paths]
    elif language == 'r':
        installer = RManager(log_file)
        arg_list = [name, version, git]
    else:
        click.echo('No installer selected')
        exit(1)
    installer.install(*arg_list)

@bus.command()
@click.option('--endpoint', '-e', help='Bus Endpoint', default='localhost')
@click.option('--port', '-p', help='Bus Port', type=click.INT, default=61613)
@click.option('--destination', '-d', help='Bus Destination', required=True)
@click.option('--selector', '-s', help='Bus Header Selector')
@click.option('--python_json_path', '-p', help='Python packages json list path', default="")
@click.option('--pip_paths', '-i', help='Pip paths', multiple=True, default=["pip"])
@click.option('--r_json_path', '-r', help='R packages json list path', default="")
@click.option('--log_file', '-f', help='LogFile')
def monitor(endpoint, port, destination, selector, python_json_path, pip_paths, r_json_path, log_file):
    installer_configs = {
        'python': {
            'json': python_json_path,
            'pip_paths': pip_paths,
            'log_file': log_file
        },       
        'r': {
            'json': r_json_path,
            'log_file': log_file

        }
    }

    BrokerConnector(endpoint, port, destination, selector, installer_configs)

installer = click.CommandCollection(sources=[bus])

if __name__ == '__main__':
    installer()


import time
import sys
import json
import stomp
from .installer import Installer


class Message(object):
    def __init__(self, j):
        self.__dict__ = json.loads(j)
        self.fields = []
        for field in self.__dict__:
            field_type = type(self.__dict__[field])
            self.fields.append((field, field_type))
            if field_type == dict:
                self.__dict__[field] = Message(json.dumps(self.__dict__[field]))
        self.original_json = j

    def __repr__(self):
        return str(self.get_fields())

    def get_fields(self):
        return dict(self.fields[:-1])

    def get_field_names(self):
        return self.get_fields().keys()

    def get_path(self, *path):
        root = self
        for level in path:
            fields = root.get_field_names()
            if level in fields:
                root = getattr(root, level)
            else:
                print('No level {} in {}'.format(level, ', '.join(path)))
                break
        if type(root) == list:
            return root
        else:
            print("Path need to be a list. Is a {}".format(str(type(root))))
            return []


    def to_json(self):
        return self.original_json

def connect_and_subscribe(conn, destination, selector):
    conn.start()
    conn.connect(wait=True)
    conn.subscribe(
        destination=destination,
        ack="auto",
        id=destination,
        headers = {'selector' : selector})

class BrokerListener(stomp.ConnectionListener):
    def __init__(self, conn, destination, selector, installer_configs):
        self.conn = conn
        self.destination = destination
        self.selector = selector
        self.installer_configs = installer_configs
        self.installer = Installer(installer_configs)

    def on_error(self, headers, message):
        print('received an error "%s"' % message)

    def on_message(self, headers, message):
        message = Message(message)
        print('received a message "%s"' % message.to_json())
        for path in self.installer_configs:
            self.installer_configs[path]['paths'] = message.get_path(*self.installer_configs[path]['json'].split('.'))
        print('paths parsed "%s"' % str(self.installer_configs[path]['paths']))
        self.installer.install(self.installer_configs)
        print('processed message and installed packages')

    def on_disconnected(self):
        print('disconnected')
        connect_and_subscribe(self.conn, self.destination, self.selector)

class BrokerConnector(object):
    def __init__(self, host, port, destination, selector, installer_configs):
        conn = stomp.Connection(
            host_and_ports=[(host, port)],
            reconnect_sleep_initial=10,
            reconnect_sleep_increase=1.618,
            reconnect_sleep_jitter=0.1,
            reconnect_sleep_max=300.0,
            reconnect_attempts_max=((24 * 60 * 60) / 300) * 2,  # Almost two days
            heartbeats=(0, 0)
        )
        conn.set_listener('broker-packager', BrokerListener(conn, destination, selector, installer_configs))
        connect_and_subscribe(conn, destination, selector)
        self.delay()
    
    def delay(self):
        while True:
            time.sleep(100000)


import subprocess
from .base import BaseManager

class PyManager(BaseManager):

    def do_install(self, package, pip_paths, *args):
        self.log_install(package, *pip_paths, *args)
        for pip_path in pip_paths:
            subprocess.call([pip_path, 'install', package, *args])

    def install(self, package, version, git, pip_paths=['pip']):
        if package:
            if git:
                self.do_install('git+{}'.format(package), pip_paths)
            elif version:           
                self.do_install('{}=={}'.format(package, version), pip_paths)
            else:
                self.do_install('{}'.format(package), pip_paths, '-U')                                        

    def install_list(self, package_list, extra_config={}):
        pip_paths = extra_config.get('pip_paths', ['pip'])
        for package_item in package_list:
            package = package_item.get('name')
            version = package_item.get('version')
            git = package_item.get('git')
            self.install(package, version, git)




from .managers.python import PyManager
from .managers.r import RManager

class Installer(object):
    def __init__(self, config={}):
        python_log_file = config.get('python', {}).get('log_file', '')
        r_log_file = config.get('r', {}).get('log_file', '')
        self.managers = {'python': PyManager(python_log_file), 'r': RManager(r_log_file)}

    def install(self, config):
        for manager_name in self.managers:
            manager = self.managers[manager_name]
            manager.install_list(config.get(manager_name, {}).get('paths', []), config.get(manager_name, {}))


import logging

class BaseManager(object):
    def __init__(self, log_file=''):
        self.logger = logging.getLogger('broker-packager')
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)
        if log_file:
            hdlr = logging.FileHandler(log_file)
            hdlr.setFormatter(formatter)
            self.logger.addHandler(hdlr) 
        self.logger.setLevel(logging.WARNING)
    
    def install(self, package, version, git, *args):
        raise NotImplementedError

    def install_list(self, package_list, extra_config={}):
        raise NotImplementedError
    
    def log_install(self, package, *args):
        self.logger.info('Installing {} with args: "{}"'.format(package, ', '.join(map(str, args))))